package org.quartz.impl;

import org.quartz.*;

import java.util.Hashtable;
import java.beans.BeanInfo;
import java.beans.PropertyDescriptor;
import java.beans.Introspector;
import java.lang.reflect.Method;

/**
 * Author: Dejan Bosanac <chubrilo@yahoo.com>
 * Date: Oct 19, 2003
 * Time: 7:53:38 PM
 *
 * Actual Scheduler XML-RPC API (modified because of the protocol restrictions)
 *
 * <code>JobDetail</code> and <code>Trigger</code> classes in method signatures are changed
 * with Hashtable structures. All the methods that returns <code>void</code> in Scheduler
 * API are now returning <code>boolean</code>.
 *
 */
public class XmlRpcScheduler {

	/**
	 * Actual instance of scheduler
	 * @see org.quartz.Scheduler
	 */
	private Scheduler scheduler;

	/**
	 * Constructor
	 * @param scheduler
	 */
	public XmlRpcScheduler(Scheduler scheduler) {
		this.scheduler = scheduler;
	}

	/**
	 * @see org.quartz.Scheduler#addJob(JobDetail, boolean)
	 */
	public boolean addJob(Hashtable jobDetails, Boolean replace)
			throws SchedulerException {
		JobDetail job = new JobDetail();
		populateObject(job, jobDetails);
		scheduler.addJob(job, replace.booleanValue());
		return true;
	}

	/**
	 * @see org.quartz.Scheduler#scheduleJob(Trigger)
	 */
	public boolean scheduleJob(Hashtable triggerDetails)
			throws SchedulerException, ClassNotFoundException {

		Trigger trigger = createTrigger(triggerDetails);
		populateObject(trigger, triggerDetails);
		scheduler.scheduleJob(trigger);

		return true;
	}

	/**
	 * @see org.quartz.Scheduler#scheduleJob(JobDetail, Trigger)
	 */
	public boolean scheduleJob(Hashtable jobDetails, Hashtable triggerDetails)
		throws SchedulerException {

		JobDetail job = new JobDetail();
		populateObject(job, jobDetails);

		Trigger trigger = createTrigger(triggerDetails);
		populateObject(trigger, triggerDetails);

		scheduler.scheduleJob(job, trigger);

		return true;
	}

	/**
	 * @see org.quartz.Scheduler#triggerJob(String, String)
	 */
	public boolean triggerJob(String name, String group)
			throws SchedulerException {
		scheduler.triggerJob(name, group);
		return true;
	}

	/**
	 * @see org.quartz.Scheduler#getSchedulerName()
	 */
	public String getSchedulerName() throws SchedulerException {
		return scheduler.getSchedulerName();
	}

	/**
	 * @see org.quartz.Scheduler#getSchedulerInstanceId()
	 */
	public String getSchedulerInstanceId() throws SchedulerException {
		return scheduler.getSchedulerInstanceId();
	}

	/**
	 * @see org.quartz.Scheduler#start()
	 */
	public boolean start() throws SchedulerException {
		scheduler.start();
		return true;
	}

	/**
	 * @see org.quartz.Scheduler#pause()
	 */
	public boolean pause() throws SchedulerException {
		scheduler.pause();
		return true;
	}

	/**
	 * @see org.quartz.Scheduler#isPaused()
	 */
	public boolean isPaused() throws SchedulerException {
		return scheduler.isPaused();
	}

	/**
	 * @see org.quartz.Scheduler#shutdown()
	 */
	public boolean shutdown() throws SchedulerException {
		scheduler.shutdown();
		return true;
	}

	/**
	 * @see org.quartz.Scheduler#shutdown(boolean)
	 */
	public boolean shutdown(boolean waitForJobsToComplete)
			throws SchedulerException {
		scheduler.shutdown(waitForJobsToComplete);
		return true;
	}

	/**
	 * @see org.quartz.Scheduler#unscheduleJob(String, String)
	 */
	public boolean unscheduleJob(String triggerName, String groupName)
			throws SchedulerException {
		return scheduler.unscheduleJob(triggerName, groupName);
	}

	/**
	 * @see org.quartz.Scheduler#deleteJob(String, String)
	 */
	public boolean deleteJob(String jobName, String groupName)
			throws SchedulerException {
		return scheduler.deleteJob(jobName, groupName);
	}

	/**
	 * @see org.quartz.Scheduler#triggerJobWithVolatileTrigger(String, String)
	 */
	public boolean triggerJobWithVolatileTrigger(String jobName, String groupName)
			throws SchedulerException {
		scheduler.triggerJobWithVolatileTrigger(jobName, groupName);
		return true;
	}

	/**
	 * @see org.quartz.Scheduler#pauseJob(String, String)
	 */
	public boolean pauseJob(String jobName, String groupName)
			throws SchedulerException {
		scheduler.pauseJob(jobName, groupName);
		return true;
	}

	/**
	 * @see org.quartz.Scheduler#pauseJobGroup(String)
	 */
	public boolean pauseJobGroup(String groupName) throws SchedulerException {
		scheduler.pauseJobGroup(groupName);
		return true;
	}

	/**
	 * @see org.quartz.Scheduler#pauseTriggerGroup(String)
	 */
	public boolean pauseTriggerGroup(String groupName) throws SchedulerException {
		scheduler.pauseTriggerGroup(groupName);
		return true;
	}

	/**
	 * @see org.quartz.Scheduler#resumeJob(String, String)
	 */
	public boolean resumeJob(String jobName, String groupName)
			throws SchedulerException {
		scheduler.resumeJob(jobName, groupName);
		return true;
	}

	/**
	 * @see org.quartz.Scheduler#resumeJobGroup(String)
	 */
	public boolean resumeJobGroup(String groupName) throws SchedulerException {
		scheduler.resumeJobGroup(groupName);
		return true;
	}

	/**
	 * @see org.quartz.Scheduler#resumeTrigger(String, String)
	 */
	public boolean resumeTrigger(String triggerName, String groupName)
			throws SchedulerException {
		scheduler.resumeTrigger(triggerName, groupName);
		return true;
	}

	/**
	 * @see org.quartz.Scheduler#resumeTriggerGroup(String)
	 */
	public boolean resumeTriggerGroup(String groupName) throws SchedulerException {
		scheduler.resumeTriggerGroup(groupName);
		return true;
	}

	/**
	 * @see org.quartz.Scheduler#pauseAll()
	 */
	public boolean pauseAll() throws SchedulerException {
		scheduler.pauseAll();
		return true;
	}

	/**
	 * @see org.quartz.Scheduler#resumeAll()
	 */
	public boolean resumeAll() throws SchedulerException {
		scheduler.resumeAll();
		return true;
	}


	/**
	 *
	 * Helper method that initializes <code>Object</code> from details given in the <code>Hashtable</code>
	 *
	 * @param object - to be populated
	 * @param details - if property of the object exists in the data map it will be set
	 * @throws SchedulerException
	 */
	private void populateObject(Object object, Hashtable details)
		throws SchedulerException {
		try {
			BeanInfo bi = Introspector.getBeanInfo(object.getClass());
			PropertyDescriptor[] pd = bi.getPropertyDescriptors();
			for (int i = 0; i < pd.length; i++) {
				if (details.containsKey(pd[i].getName())) {
					Object argument = details.get(pd[i].getName());
					Method method = pd[i].getWriteMethod();
					Class[] methodParams = method.getParameterTypes();
					Object[] arguments = new Object[1];
					if (methodParams[0].getName() == "java.lang.Class")
						arguments[0] = Class.forName((String)argument);
					else
						arguments[0] = argument;

					method.invoke(object, arguments);
				}
			}
		} catch (Exception ex) {
			throw new SchedulerException(ex);
		}
	}

	/**
	 * Helper method that creates appropriate <code>Trigger</code> subclass
	 * @param triggerDetails - if "className" exists creates appropriate object
	 * @return
	 * @throws SchedulerException
	 */
	private Trigger createTrigger(Hashtable triggerDetails)
		throws SchedulerException {

		Trigger trigger;
		if (!triggerDetails.containsKey("className"))
			throw new SchedulerException(new NullPointerException("\"className\" parameter for trigger must be set!"));
		try {
			trigger = (Trigger)Class.forName((String)triggerDetails.get("className")).newInstance();
		} catch (Exception ex) {
			throw new SchedulerException(ex);
		}

		return trigger;
	}


}